"use strict";

var _interopRequireWildcard = require("@babel/runtime/helpers/interopRequireWildcard");

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

exports.__esModule = true;
exports.shouldShowNodeByExpanded = shouldShowNodeByExpanded;
exports.flattenTree = flattenTree;
exports.getNodeParentKeys = getNodeParentKeys;
exports.shouldDisplay = shouldDisplay;
exports.getVirtualLisHeight = getVirtualLisHeight;

var _extends2 = _interopRequireDefault(require("@babel/runtime/helpers/extends"));

var _trim2 = _interopRequireDefault(require("lodash/trim"));

var _intersection2 = _interopRequireDefault(require("lodash/intersection"));

var React = _interopRequireWildcard(require("react"));

var _utils = require("rsuite-utils/lib/utils");

var SEARCH_BAR_HEIGHT = 48;
var MENU_PADDING = 12;
/**
 * 判断当前节点是否应该显示
 * @param {*} expandItemValues
 * @param {*} parentKeys
 */

function shouldShowNodeByExpanded(expandItemValues, parentKeys) {
  if (expandItemValues === void 0) {
    expandItemValues = [];
  }

  if (parentKeys === void 0) {
    parentKeys = [];
  }

  var intersectionKeys = (0, _intersection2.default)(expandItemValues, parentKeys);

  if (intersectionKeys.length === parentKeys.length) {
    return true;
  }

  return false;
}
/**
 * 拍平树结构为数组
 * @param {*} tree
 * @param {*} executor
 * @param {*} props
 */


function flattenTree(tree, executor, props) {
  var childrenKey = props.childrenKey;
  var flattenData = [];

  var traverse = function traverse(data) {
    data.forEach(function (d, index) {
      var node = executor(d, index);
      flattenData.push((0, _extends2.default)({}, node));

      if (d[childrenKey]) {
        traverse(d[childrenKey]);
      }
    });
  };

  traverse(tree);
  return flattenData;
}
/**
 * 获取当前节点所有的祖先节点
 * @param {*} node
 */


function getNodeParentKeys(node, props) {
  var valueKey = props.valueKey;
  var parentKeys = [];

  var traverse = function traverse(node) {
    if (node && node.parentNode) {
      traverse(node.parentNode);
      parentKeys.push(node.parentNode[valueKey]);
    }
  };

  traverse(node);
  return parentKeys;
}
/**
 * 判断当前节点是否显示
 * @param {*} label
 * @param {*} searchKeyword
 */


function shouldDisplay(label, searchKeyword) {
  if (!(0, _trim2.default)(searchKeyword)) {
    return true;
  }

  var keyword = searchKeyword.toLocaleLowerCase();

  if (typeof label === 'string') {
    return label.toLocaleLowerCase().indexOf(keyword) >= 0;
  } else if (React.isValidElement(label)) {
    var nodes = (0, _utils.reactToString)(label);
    return nodes.join('').toLocaleLowerCase().indexOf(keyword) >= 0;
  }

  return false;
}
/**
 * 获取 VirtualList 的高度
 * @param {*} inline
 * @param {*} height
 */


function getVirtualLisHeight(inline, height) {
  if (height === void 0) {
    height = 0;
  }

  return inline ? height - MENU_PADDING * 2 : height - SEARCH_BAR_HEIGHT - MENU_PADDING * 2;
}